local assets=
{
	Asset("ANIM", "anim/obsidianboomerang.zip"),
	Asset("ANIM", "anim/swap_obsidianboomerang.zip"),
    Asset( "IMAGE", "images/inventoryimages/obsidianboomerang.tex" ),
    Asset( "ATLAS", "images/inventoryimages/obsidianboomerang.xml" ),
}



    
local prefabs =
{
}

local function OnFinished(inst)
    inst.AnimState:PlayAnimation("used")
    inst:ListenForEvent("animover", function() inst:Remove() end)
end

local function OnEquip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_obsidianboomerang", "swap_boomerang")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function OnHitOwner(inst)
    inst.components.floatable:SetAnimationFromPosition()
end

local function OnUnequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function OnThrown(inst, owner, target)
    if target ~= owner then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_throw")
    end
    inst.AnimState:PlayAnimation("spin_loop", true)
end

local function OnCaught(inst, catcher)
    if catcher then
        if catcher.components.inventory then
            if inst.components.equippable and not catcher.components.inventory:GetEquippedItem(inst.components.equippable.equipslot) then
				catcher.components.inventory:Equip(inst)
			else
                catcher.components.inventory:GiveItem(inst)
            end
            catcher:PushEvent("catch")
        end
    end
end

local function ReturnToOwner(inst, owner)
    if owner and not (inst.components.finiteuses and inst.components.finiteuses:GetUses() < 1) then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_return")
		 local cruxtarget = FindEntity(owner, 25, function(guy) 
            if guy.components.combat and guy.components.health and not guy.components.health:IsDead() and not guy:HasTag("player") then
               return guy.components.combat.target == owner or owner.components.combat.target == guy or guy:HasTag("monster")
            end
        end )
		if cruxtarget then
           inst.components.projectile:Throw(owner, cruxtarget)
        else
           inst.components.projectile:Throw(owner, owner)
        end

    end
end

local function OnHit(inst, owner, target)
    if owner == target then
        OnHitOwner(inst)
		if OBSIDIANBOOMERANGPUNISHMENT then
            local coordinate = target:GetPosition()
            GetSeasonManager():DoLightningStrike(coordinate)
		end
    else
        ReturnToOwner(inst, owner)
    end
    local impactfx = SpawnPrefab("impact")
    if impactfx then
	    local follower = impactfx.entity:AddFollower()
	    follower:FollowSymbol(target.GUID, target.components.combat.hiteffectsymbol, 0, 0, 0 )
        impactfx:FacePoint(inst.Transform:GetWorldPosition())
    end
end


local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst .entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)
    
    anim:SetBank("boomerang")
    anim:SetBuild("obsidianboomerang")
    anim:PlayAnimation("idle")
    anim:SetRayTestOnBB(true);
    
    MakeInventoryFloatable(inst, "idle_water", "idle")
    
    inst:AddTag("projectile")
    inst:AddTag("thrown")
    
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.BOOMERANG_DAMAGE)
    inst.components.weapon:SetRange(TUNING.BOOMERANG_DISTANCE, TUNING.BOOMERANG_DISTANCE+2)
    -------
    
    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(TUNING.BOOMERANG_USES*5)
    inst.components.finiteuses:SetUses(TUNING.BOOMERANG_USES*5)
    
    inst.components.finiteuses:SetOnFinished(OnFinished)

    inst:AddComponent("inspectable")
	
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.imagename = "obsidianboomerang"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/obsidianboomerang.xml" 
    inst.components.inventoryitem.foleysound = "dontstarve/movement/foley/jewlery"
	
    inst:AddComponent("projectile")
    inst.components.projectile:SetSpeed(10)
    inst.components.projectile:SetCanCatch(true)
    inst.components.projectile:SetOnThrownFn(OnThrown)
    inst.components.projectile:SetOnHitFn(OnHit)
    inst.components.projectile:SetOnMissFn(ReturnToOwner)
    inst.components.projectile:SetOnCaughtFn(OnCaught)
    inst.components.projectile:SetLaunchOffset(Vector3(0, 0.2, 0))
	
	inst:ListenForEvent("onthrown", function(inst)
        local pt = inst:GetPosition()
        inst.Transform:SetPosition(pt.x, 2, pt.z)
        inst.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_throw")
    end)
	
    
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquip)
    inst.components.equippable:SetOnUnequip(OnUnequip)
	
	STRINGS.NAMES.OBSIDIANBOOMERANG = "Obsidian Boomerang"
    STRINGS.RECIPE_DESC.OBSIDIANBOOMERANG = "Good Job!"
    
    return inst
end

return Prefab( "common/inventory/obsidianboomerang", fn, assets) 
